#!/usr/bin/perl -w

#
#  generate_base_ipv4_hitlist.pl
#  Copyright (C) 2010 by USC/ISI
#  $Id$
#  
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License,
#  version 2, as published by the Free Software Foundation.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#  
#  
#  This code was originally written by Xun Fan <xunfan@isi.edu>.
#

#
use strict;

use Fsdb::IO::Reader;
use Fsdb::IO::Writer;
use Getopt::Long;
use Pod::Usage;

Getopt::Long::Configure ("bundling");

my $usagetxt = "$0 usage:\n$0 (input_fsdb_file) <output_directory>\n";

pod2usage($usagetxt) if ($#ARGV >= 0 && $ARGV[0] eq '-?');
pod2usage($usagetxt) if ($#ARGV < 0);

# base IANA reserved blocks
# for customized blacklist, please
# start after 203.0.113.0|24.
my @blacklist = ( 
	"169.254.0.0|16" ,
	"172.16.0.0|12" ,
	"192.0.2.0|24" ,
	"192.88.99.0|24" ,
	"192.168.0.0|16" ,
	"192.0.0.0|24" ,
	"198.18.0.0|15" ,
	"198.51.100.0|24" ,
	"203.0.113.0|24" 
);

#get option
my ($prog) = $0;
my ($input_file) = $ARGV[0];

my $output_dir;
if(!defined($ARGV[1])){
	$output_dir = "\.";
}else{
	$output_dir = $ARGV[1];
}

#open file to read with fsdb
my $fsdb_in = new Fsdb::IO::Reader(-file => $input_file);

# read a row
my @input_arow;
sub read_rows{
        $fsdb_in->read_row_to_aref(\@input_arow) or return undef;
	return ($input_arow[0], $input_arow[1], $input_arow[2], $input_arow[3], $input_arow[4], $input_arow[5]);
}

my @line;
my $line_count = 0;

my @BLprefix = ();
my @BLlength = ();
my $BLelementscnt = 0;

# divide blacklist entry into hex ip and prefix length
sub BlacklistInit{
	my $i = 0;
	my $hexpre;
	my $hexlen;
	my ($pre, $len);
	foreach(@blacklist){
		($pre, $len) = split(/\|/, $_);
		my ($a, $b, $c, $d) = split(/\./, $pre);
		$hexpre = $a * (2**24) + $b * (2**16) + $c * (2**8) + $d;
		push(@BLprefix, $hexpre);
		$hexlen = ~(2**(32-$len) -1);
		push(@BLlength, $hexlen);
		$i++;
	}
	$BLelementscnt = @BLprefix;
}

# hex IP address to string IP address
sub hexiptoip{
	my ($hexip) = @_;
	my $ip1 = hex(substr($hexip, 0, 2));
	my $ip2 = hex(substr($hexip, 2, 2));
	my $ip3 = hex(substr($hexip, 4, 2));
	my $ip4 = hex(substr($hexip, 6, 2));
	return "$ip1.$ip2.$ip3.$ip4";
}

#check if the IP address falls into Blacklist
sub checkevil{
	my ($addr) = @_;
	my $i = 0;
	for ($i = 0; $i < $BLelementscnt; $i++)
	{
		if(($addr & $BLlength[$i]) == $BLprefix[$i]){
			return 1;
		}
	}
	return 0;
}

#initialize blacklist
BlacklistInit();


for(;;)
{
	my ($pre, $dest, $date, $whois, $status, $probing) = &read_rows;
	last if (!defined($pre)); # EOF

	next if ($probing eq "false");

	# get prefix
	my ($firstOct, $blockSize) = split(/\//, $pre);

	# output variables
	my ($prefix, $hex_ip, $score, $ip, $hex_ip_print);
	my $hex_prefix;

#=pod
	# Open output file
	$firstOct = sprintf("%03d", $firstOct);
	open (OUT, ">$output_dir\/$firstOct") || die "can't write to file $output_dir\/$firstOct";
	my $output_stream = new Fsdb::IO::Writer(-fh => \*OUT, -fscode => 't', -cols => [qw(prefix hex_ip score ip)]);


	for (my $i=0; $i<256; $i++)
	{
		for (my $j=0; $j<256; $j++)
		{
			$hex_prefix = $firstOct * 2**24 + $i * 2**16 + $j * 2**8;
			$hex_ip = $hex_prefix + 1;
			$score = -40;
			next if(checkevil($hex_ip) > 0);
			$prefix = sprintf("%08x", $hex_prefix);
			$hex_ip_print = sprintf("%08x", $hex_ip);
			$ip = hexiptoip($hex_ip_print);
			$output_stream->write_row($prefix, $hex_ip_print, $score, $ip);
		}
	}


	close(OUT);
#=cut
}

$fsdb_in->close;

